#include <fstream>
#include <string>
#include <vector>
#include <map>
#include <queue>
#include <algorithm>
#include <cassert>

const std::string FILENAME("corsa");

std::ifstream cin(FILENAME + ".in");
std::ofstream cout(FILENAME + ".out");


enum EventType {RainBegin, RainEnd, SnowBegin, SnowEnd, Query};

struct Event {
    EventType type;
    int index;

    Event(){}
    Event(EventType type, int index) 
        : type(type), index(index) 
    {}
};

const int default_speed = 100;
const int MaxQ = 1e5 + 5;

int main() {

    long long n;
    int nr_rain, nr_snow, nr_query;

    cin >> n;
    cin >> nr_snow >> nr_rain >> nr_query; 
    
    assert(n > 0LL);
    assert(n <= 1e10);
    assert(nr_snow >= 0);
    assert(nr_snow <= 100000);
    assert(nr_rain >= 0);
    assert(nr_rain <= 100000);
    assert(nr_query > 0);
    assert(nr_query <= 100000);

    std::vector<long long> rain_coef(nr_rain), snow_coef(nr_snow), query_ans(nr_query);
    std::map<long long,std::vector<Event>> events;

    for (int i = 0; i < nr_snow; ++i) {
        long long l, r;
        cin >> l >> r >> snow_coef[i]; 
        //cout<<l<<' '<<r<<'\n';
        assert(l > 0LL);
        assert(l < r);
        assert(r <= n);
        assert(snow_coef[i] > 0);
        assert(snow_coef[i] <= 100000);

        events[l].emplace_back(SnowBegin, i);
        events[r].emplace_back(SnowEnd, i);
    }

    for (int i = 0; i < nr_rain; ++i) {
        long long l, r;
        cin >> l >> r >> rain_coef[i]; 
        //cout<<l<<' '<<r<<'\n';
        assert(l > 0LL);
        assert(l < r);
        assert(r <= n);
        assert(rain_coef[i] > 0);
        assert(rain_coef[i] <= 100000);

        events[l].emplace_back(RainBegin, i);
        events[r].emplace_back(RainEnd, i);
    }

    for (int i = 0; i < nr_query; ++i) {
        long long p;
        cin >> p;

        assert(p > 0);
        assert(p <= n);

        events[p].emplace_back(Query, i);
    }

    std::priority_queue<std::pair<int,int>> active_rains;
    std::vector<bool> out_of_queue(nr_rain, false);

    long long current_time = 0;
    long long last_position = 1;
    long long current_rain_limit = default_speed;
    long long current_snow_limit = default_speed;
    long long current_speed = default_speed;
    for (const auto& it : events) {
        long long current_position = it.first;
        std::vector<Event> current_events = it.second;
        long long delta_distance = current_position - last_position; 
        current_time += 1LL * delta_distance * current_speed;        
        for (const Event& event : current_events) {
            switch (event.type) {
                case Query: 
                    query_ans[event.index] = current_time;
                    break;
                case SnowBegin:
                    current_snow_limit += snow_coef[event.index];
                    break;
                case SnowEnd:
                    current_snow_limit -= snow_coef[event.index];
                    break;
                case RainBegin:
                    active_rains.push({rain_coef[event.index], event.index});
                    break;
                case RainEnd:
                    out_of_queue[event.index] = true;
                    break;
            }
        }
        while (!active_rains.empty() && out_of_queue[active_rains.top().second]) {
            active_rains.pop();
        }

        current_rain_limit = active_rains.empty() ? default_speed : active_rains.top().first;

        current_speed = std::max(current_rain_limit, current_snow_limit);
        last_position = current_position;
        //cout<<current_position<<' '<<' '<<current_speed<<' '<<current_time<<' '<< current_snow_limit<<' '<<current_rain_limit<<'\n';
    }

    for (int i = 0; i < nr_query; ++i) {
        assert(query_ans[i] >= 0);
        cout << query_ans[i] << '\n';
    }
}